/*
 * Copyright (c) 2010-2024. Institut Pasteur.
 *
 * This file is part of Icy.
 * Icy is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Icy is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Icy. If not, see <https://www.gnu.org/licenses/>.
 */

package plugins.spop.rotation3D;

import org.bioimageanalysis.icy.common.collection.array.Array1DUtil;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginIcon;
import org.bioimageanalysis.icy.extension.plugin.annotation_.IcyPluginName;
import org.bioimageanalysis.icy.model.OMEUtil;
import org.bioimageanalysis.icy.model.image.IcyBufferedImage;
import org.bioimageanalysis.icy.model.sequence.Sequence;
import org.jetbrains.annotations.NotNull;
import plugins.adufour.blocks.lang.Block;
import plugins.adufour.blocks.util.VarList;
import plugins.adufour.ezplug.EzPlug;
import plugins.adufour.ezplug.EzVarSequence;
import plugins.adufour.ezplug.EzVarText;

@IcyPluginName("Stack Rotation")
@IcyPluginIcon(path = "/stack-rotation.png")
public class StackRotation extends EzPlug implements Block {
    EzVarSequence input = new EzVarSequence("Input");
    EzVarText type = new EzVarText("Type", new String[]{"FrontToRight", "FrontToTop"}, false);
    // VarSequence output = new VarSequence("Output", false);

    EzVarSequence output = new EzVarSequence("Output");

    @Override
    public void initialize() {
        super.addEzComponent(input);
        super.addEzComponent(type);
        // super.addOutput(output);
        super.setTimeDisplay(true);

    }

    @Override
    public void execute() {
        final Sequence sequence = input.getValue(true);
        Sequence seq_out = new Sequence(OMEUtil.createOMEXMLMetadata(sequence.getOMEXMLMetadata()));
        if (type.getValue().equalsIgnoreCase("FrontToRight")) {
            seq_out = FrontToRight(sequence);
            seq_out.setName("FrontToRight");
        }
        if (type.getValue().equalsIgnoreCase("FrontToTop")) {
            seq_out = FrontToTop(sequence);
            seq_out.setName("FrontToTop");

        }

        if (getUI() != null) addSequence(seq_out);
        output.setValue(seq_out);
    }

    public static @NotNull Sequence FrontToRight(final @NotNull Sequence sequence) {
        int z; // = 0;

        final Sequence seq = new Sequence();

        final int dim_x = sequence.getSizeX();
        final int dim_y = sequence.getSizeY();
        final int dim_z = sequence.getSizeZ();
        final int dim_t = sequence.getSizeT();
        final int dim_c = sequence.getSizeC();
        final double[][] tab = new double[dim_z][dim_x * dim_y];
        final double[][][] tab_r = new double[dim_c][dim_x][dim_y * dim_z];

        for (int t = 0; t < dim_t; t++) {
            for (int c = 0; c < dim_c; c++) {
                for (z = 0; z < dim_z; z++) {
                    tab[z] = Array1DUtil.arrayToDoubleArray(sequence.getDataXY(t, z, c), false);
                }
                for (z = 0; z < dim_z; z++)
                    for (int y = 0; y < dim_y; y++)
                        for (int x = 0; x < dim_x; x++) {
                            tab_r[c][x][y * dim_z + z] = tab[z][y * dim_x + x];
                        }
            }

            for (z = 0; z < dim_x; z++) {
                final IcyBufferedImage Icy = new IcyBufferedImage(dim_z, dim_y, dim_c, sequence.getDataType());
                for (int c = 0; c < dim_c; c++)
                    Icy.setDataXY(c, tab_r[c][z]);
                seq.setImage(t, z, Icy);
            }
        }

        System.gc();
        return seq;
    }

    public static @NotNull Sequence FrontToTop(final @NotNull Sequence sequence) {
        int z; // = 0;

        final Sequence seq = new Sequence();

        final int dim_x = sequence.getSizeX();
        final int dim_y = sequence.getSizeY();
        final int dim_z = sequence.getSizeZ();
        final int dim_t = sequence.getSizeT();
        final int dim_c = sequence.getSizeC();
        final double[][] tab = new double[dim_z][dim_x * dim_y];
        final double[][][] tab_r = new double[dim_c][dim_y][dim_x * dim_z];

        for (int t = 0; t < dim_t; t++) {
            for (int c = 0; c < dim_c; c++) {
                for (z = 0; z < dim_z; z++) {
                    tab[z] = Array1DUtil.arrayToDoubleArray(sequence.getDataXY(t, z, c), false);
                }
                for (z = 0; z < dim_z; z++)
                    for (int y = 0; y < dim_y; y++)
                        if (dim_x >= 0) System.arraycopy(
                                tab[z],
                                y * dim_x, // + 0,
                                tab_r[c][y],
                                z * dim_x, // + 0,
                                dim_x);
            }

            for (z = 0; z < dim_y; z++) {
                final IcyBufferedImage Icy = new IcyBufferedImage(dim_x, dim_z, dim_c, sequence.getDataType());
                for (int c = 0; c < dim_c; c++)
                    Icy.setDataXY(c, tab_r[c][z]);
                seq.setImage(t, z, Icy);
            }
        }

        System.gc();

        return seq;

    }

    @Override
    public void clean() {
    }

    @Override
    public void declareInput(final @NotNull VarList inputMap) {
        inputMap.add("input", input.getVariable());
        inputMap.add("type", type.getVariable());
    }

    @Override
    public void declareOutput(final @NotNull VarList outputMap) {
        outputMap.add("output", output.getVariable());
    }
}
